{------------------------------------------------------------------------------
  plsDialogs.pas

  Purpose:    Language supported dialogs (messagedlg, taskmessagedlg, ...)
              (part of Precision Language Suite for VCL)

  The initial author of the source code and the copyright holder is CodeGear,
  Copyright (c) 1995-2008 CodeGear

  The localization support enhancements are written by Precision software & consulting
              e-mail: info@be-precision.com
              web: http://www.be-precision.com

  The enhancements source code is given as is. The author is not responsible
  for any possible damage done due to the use of this code.

------------------------------------------------------------------------------
  History:

  - Version: 2.2.4.19
    * added - Support for "elevation required" shields (contribution by ouiouioui, www.toutenvrac.org)
------------------------------------------------------------------------------}

unit plsDialogs;

{$I plsCompilers.inc}

interface

uses
  Consts,
  Classes,
  Dialogs;

var
  lng_MsgDlgCaptions:array[TMsgDlgType] of string = (
    SMsgDlgWarning,SMsgDlgError,SMsgDlgInformation,SMsgDlgConfirm,'');

  lng_ButtonCaptions: array[TMsgDlgBtn] of string = (
    SMsgDlgYes, SMsgDlgNo, SMsgDlgOK, SMsgDlgCancel, SMsgDlgAbort,
    SMsgDlgRetry, SMsgDlgIgnore, SMsgDlgAll, SMsgDlgNoToAll, SMsgDlgYesToAll,
    SMsgDlgHelp{$IFDEF D12}, SMsgDlgClose{$ENDIF D12});

// procedure LanguageChanged must be called on every language change/load (see language identifiers in this procedure to add to your language files)
procedure LanguageChanged;
// procedure MessageDlgLM supports standard MessageDlg (WinXP/2000) and TaskMessageDlg(Vista/7) routines - localized
function MessageDlgLM(const MsgCaption, Msg: string; DlgType: TMsgDlgType; Buttons:TMsgDlgButtons = [mbOK];
  DefaultButton:TMsgDlgBtn=mbOK; X:Integer=-1; Y:Integer=-1; HelpCtx:Longint=0; const HelpFileName:string='';
  BtnElevationRequired: TMsgDlgButtons = []): Integer;
// procedure InputQueryLM supports standard InputQuery routines - localized
function InputQueryLM(const ACaption, APrompt: string;
  var Value: string; sItems:TStrings=nil; Fixed:Boolean=False): Boolean;
// procedure InputQueryMemoLM supports multiline text InputQuery - localized
function InputQueryMemoLM(const ACaption, APrompt: string;
  var Value: string; mWidth:Integer=232; mHeight:Integer=232): Boolean;

implementation

uses
  Windows, SysUtils, MultiMon, {$IFDEF D7}HelpIntfs, Themes, {$ENDIF}Controls, Forms, StdCtrls,
  plsLangMan, Graphics;

const
  _ButtonNames: array[TMsgDlgBtn] of string = (
    'Yes', 'No', 'OK', 'Cancel', 'Abort', 'Retry', 'Ignore', 'All', 'NoToAll',
    'YesToAll', 'Help'{$IFDEF D12}, 'Close'{$ENDIF D12}); // do not localize

procedure LanguageChanged;
begin
  with LanguageManager do
  begin
    lng_MsgDlgCaptions[mtWarning]:=LangText('lng_MsgDlgWarning',SMsgDlgWarning);
    lng_MsgDlgCaptions[mtError]:=LangText('lng_MsgDlgError',SMsgDlgError);
    lng_MsgDlgCaptions[mtInformation]:=LangText('lng_MsgDlgInformation',SMsgDlgInformation);
    lng_MsgDlgCaptions[mtConfirmation]:=LangText('lng_MsgDlgConfirm',SMsgDlgConfirm);

    lng_ButtonCaptions[mbYes]:=LangText('lng_MsgBtnYes',SMsgDlgYes);
    lng_ButtonCaptions[mbNo]:=LangText('lng_MsgBtnNo',SMsgDlgNo);
    lng_ButtonCaptions[mbOK]:=LangText('lng_MsgBtnOK',SMsgDlgOK);
    lng_ButtonCaptions[mbCancel]:=LangText('lng_MsgBtnCancel',SMsgDlgCancel);
    lng_ButtonCaptions[mbAbort]:=LangText('lng_MsgBtnAbort',SMsgDlgAbort);
    lng_ButtonCaptions[mbRetry]:=LangText('lng_MsgBtnRetry',SMsgDlgRetry);
    lng_ButtonCaptions[mbIgnore]:=LangText('lng_MsgBtnIgnore',SMsgDlgIgnore);
    lng_ButtonCaptions[mbAll]:=LangText('lng_MsgBtnAll',SMsgDlgAll);
    lng_ButtonCaptions[mbNoToAll]:=LangText('lng_MsgBtnNoToAll',SMsgDlgNoToAll);
    lng_ButtonCaptions[mbYesToAll]:=LangText('lng_MsgBtnYesToAll',SMsgDlgYesToAll);
    lng_ButtonCaptions[mbHelp]:=LangText('lng_MsgBtnHelp',SMsgDlgHelp);
    {$IFDEF D12}
    lng_ButtonCaptions[mbClose]:=LangText('lng_MsgBtnClose',SMsgDlgClose);
    {$ENDIF D12}
  end;
end;

{$IFDEF D12}
{ TaskDialog based message dialog; requires Windows Vista or later }

type
  TSimpleTaskMessageDialog = class(TCustomTaskDialog)
  private
    FHelpFile: string;
    FParentWnd: HWND;
    FPosition: TPoint;
  strict protected
    procedure DoOnButtonClicked(AModalResult: Integer; var CanClose: Boolean); override;
    procedure DoOnDialogCreated; override;
    procedure DoOnHelp; override;
  public
    function Execute(ParentWnd: HWND): Boolean; overload; override;
    property HelpFile: string read FHelpFile write FHelpFile;
    property Position: TPoint read FPosition write FPosition;
  end;

const
  tdbHelp = -1;

procedure TSimpleTaskMessageDialog.DoOnButtonClicked(AModalResult: Integer;
  var CanClose: Boolean);
begin
  if AModalResult = tdbHelp then
  begin
    CanClose := False;
    DoOnHelp;
  end;
end;

procedure TSimpleTaskMessageDialog.DoOnDialogCreated;
var
  Rect: TRect;
  LX, LY: Integer;
  LHandle: HMONITOR;
  LMonitorInfo: TMonitorInfo;
begin
  LX := Position.X;
  LY := Position.Y;
  LHandle := MonitorFromWindow(FParentWnd, MONITOR_DEFAULTTONEAREST);
  LMonitorInfo.cbSize := SizeOf(LMonitorInfo);
  if GetMonitorInfo(LHandle, @LMonitorInfo) then
    with LMonitorInfo do
    begin
      GetWindowRect(Handle, Rect);
      if LX < 0 then
        LX := ((rcWork.Right - rcWork.Left) - (Rect.Right - Rect.Left)) div 2;
      if LY < 0 then
        LY := ((rcWork.Bottom - rcWork.Top) - (Rect.Bottom - Rect.Top)) div 2;
      Inc(LX, rcWork.Left);
      Inc(LY, rcWork.Top);
      SetWindowPos(Handle, 0, LX, LY, 0, 0, SWP_NOACTIVATE or SWP_NOSIZE or SWP_NOZORDER);
    end;
end;

procedure TSimpleTaskMessageDialog.DoOnHelp;
var
  LHelpFile: string;
  LHelpSystem: IHelpSystem;
begin
  if HelpContext <> 0 then
  begin
    if FHelpFile = '' then
      LHelpFile := Application.HelpFile
    else
      LHelpFile := HelpFile;
    if HelpIntfs.GetHelpSystem(LHelpSystem) then
    try
      LHelpSystem.Hook(Application.Handle, LHelpFile, HELP_CONTEXT, HelpContext);
    except
      on E: Exception do
        ShowHelpException(E);
    end;
  end;
end;

function TSimpleTaskMessageDialog.Execute(ParentWnd: HWND): Boolean;
begin
  FParentWnd := ParentWnd;
  Result := inherited Execute(ParentWnd);
end;
{$ENDIF D12}

function MessageDlgLM(const MsgCaption, Msg: string; DlgType: TMsgDlgType; Buttons:TMsgDlgButtons = [mbOK];
  DefaultButton:TMsgDlgBtn=mbOK; X:Integer=-1; Y:Integer=-1; HelpCtx:Longint=0; const HelpFileName:string='';
  BtnElevationRequired: TMsgDlgButtons = []): Integer;
{$IFDEF D12}
const
  IconMap: array[TMsgDlgType] of TTaskDialogIcon = (tdiWarning, tdiError, tdiInformation, tdiInformation, tdiNone);
  LModalResults: array[TMsgDlgBtn] of Integer = (mrYes, mrNo, mrOk, mrCancel,
    mrAbort, mrRetry, mrIgnore, mrAll, mrNoToAll, mrYesToAll, tdbHelp, mrClose);
var
  DlgBtn: TMsgDlgBtn;
  LTaskDialog: TSimpleTaskMessageDialog;
  Btn: TTaskDialogBaseButtonItem;
{$ENDIF D12}
var
  MessageDialog:TForm;
  offs:Integer;
  B:TMsgDlgBtn;
  LButton:TButton;
  LbMsg:TLabel;
begin
  {$IFDEF D12}
  if (Win32MajorVersion >= 6) and ThemeServices.ThemesEnabled then
  begin
    LTaskDialog := TSimpleTaskMessageDialog.Create(nil);
    try
      // Assign buttons
      for DlgBtn := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
        if DlgBtn in Buttons then
        begin
          Btn := LTaskDialog.Buttons.Add;
          with TTaskDialogButtonItem(Btn) do
          begin
            Caption := lng_ButtonCaptions[DlgBtn];
            Default := DlgBtn = DefaultButton;
            ModalResult := LModalResults[DlgBtn];
            ElevationRequired := DlgBtn in BtnElevationRequired;
          end;
        end;
      // Set dialog properties
      with LTaskDialog do
      begin
        if DlgType <> mtCustom then
          Caption := lng_MsgDlgCaptions[DlgType]
        else
          Caption := Application.Title;
        CommonButtons := [];
        if Application.UseRightToLeftReading then
          Flags := Flags + [tfRtlLayout];
        HelpContext := HelpCtx;
        HelpFile := HelpFileName;
        if DlgType=mtConfirmation then
        begin
          CustomMainIcon.Handle := LoadIcon(0,IDI_QUESTION);
          if CustomMainIcon.Handle<>0 then
            LTaskDialog.Flags:=LTaskDialog.Flags+[tfUseHiconMain]
          else
            MainIcon :=  IconMap[DlgType]
        end
        else
          MainIcon :=  IconMap[DlgType];
        Position := Point(X, Y);
        Text := Msg;
        if Length(MsgCaption)=0 then
          Title := Application.Title
        else
          Title := MsgCaption;
      end;

      // Show dialog and return result
      Result := mrNone;
      if LTaskDialog.Execute then
        Result := LTaskDialog.ModalResult;
    finally
      LTaskDialog.Free;
    end;
  end
  else
  {$ENDIF D12}
  begin
    MessageDialog:=CreateMessageDialog(Msg, DlgType, Buttons{$IFDEF D12}, DefaultButton{$ENDIF D12});
    try
      LbMsg := TLabel(MessageDialog.FindComponent('message'));
      if Assigned(LbMsg) then
        LbMsg.ShowAccelChar:=False;
      MessageDialog.HelpContext := HelpCtx;
      MessageDialog.HelpFile := HelpFileName;
      if X >= 0 then MessageDialog. Left := X;
      if Y >= 0 then MessageDialog.Top := Y;
      if (Y < 0) and (X < 0) then MessageDialog.Position := poScreenCenter;
      // localize Dialog caption
      if Length(MsgCaption)=0 then
        MessageDialog.Caption:=Application.Title
      else
        MessageDialog.Caption:=MsgCaption;
      if MessageDialog.Width<300 then
      begin
        offs:=150-(MessageDialog.Width div 2);
        MessageDialog.Width:=300;
      end
      else
        offs:=0;
      // localize buttons
      for B := Low(TMsgDlgBtn) to High(TMsgDlgBtn) do
        if B in Buttons then
        begin
          LButton := TButton(MessageDialog.FindComponent(_ButtonNames[B]));
          if Assigned(LButton) then
          begin
            LButton.Caption:=lng_ButtonCaptions[B];
            LButton.Left:=LButton.Left+offs;
          end;
        end;
      Result := MessageDialog.ShowModal;
    finally
      MessageDialog.Free;
    end;
  end;
end;

function _GetAveCharSize(Canvas: TCanvas): TPoint;
var
  I: Integer;
  Buffer: array[0..51] of Char;
begin
  for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
  for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
  GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
  Result.X := Result.X div 52;
end;

function InputQueryLM(const ACaption, APrompt: string;
  var Value: string; sItems:TStrings=nil; Fixed:Boolean=False): Boolean;
var
  Form: TForm;
  Prompt: TLabel;
  Edit: TEdit;
  cEdit: TComboBox;
  i:Integer;
  DialogUnits: TPoint;
  ButtonTop, ButtonWidth, ButtonHeight: Integer;
begin
  Result := False;
  Form := TForm.Create(Application);
  with Form do
    try
      Canvas.Font := Font;
      DialogUnits := _GetAveCharSize(Canvas);
      BorderStyle := bsDialog;
      Caption := ACaption;
      ClientWidth := MulDiv(248, DialogUnits.X, 4);
      {$IFDEF D12}
      PopupMode := pmAuto;
      {$ENDIF D12}
      Position := poScreenCenter;
      Prompt := TLabel.Create(Form);
      with Prompt do
      begin
        Parent := Form;
        Caption := APrompt;
        Left := MulDiv(8, DialogUnits.X, 4);
        Top := MulDiv(8, DialogUnits.Y, 8);
        Constraints.MaxWidth := MulDiv(232, DialogUnits.X, 4);
        ShowAccelChar:=False;
        WordWrap := True;
      end;
      cEdit:=nil;
      Edit:=nil;
      if Assigned(sItems) then
      begin
        cEdit := TComboBox.Create(Form);
        with cEdit do
        begin
          Parent := Form;
          Left := Prompt.Left;
          Top := Prompt.Top + Prompt.Height + 5;
          Width := MulDiv(232, DialogUnits.X, 4);
          MaxLength := 255;
          Items.Assign(sItems);
          if Fixed then
          begin
            Style:=csDropDownList;
            i:=Items.IndexOf(Value);
            if i>=0 then
              ItemIndex:=i
            else
            if Items.Count>0 then
              ItemIndex:=0;
          end
          else
            Text := Value;
          SelectAll;
        end;
        ButtonTop := cEdit.Top + cEdit.Height + 15;
      end
      else
      begin
        Edit := TEdit.Create(Form);
        with Edit do
        begin
          Parent := Form;
          Left := Prompt.Left;
          Top := Prompt.Top + Prompt.Height + 5;
          Width := MulDiv(232, DialogUnits.X, 4);
          MaxLength := 255;
          Text := Value;
          SelectAll;
        end;
        ButtonTop := Edit.Top + Edit.Height + 15;
      end;

      ButtonWidth := MulDiv(50, DialogUnits.X, 4);
      ButtonHeight := MulDiv(14, DialogUnits.Y, 8);
      with TButton.Create(Form) do
      begin
        Parent := Form;
        Caption := lng_ButtonCaptions[mbOK];
        ModalResult := mrOk;
        Default := True;
        SetBounds(MulDiv(68, DialogUnits.X, 4), ButtonTop, ButtonWidth, ButtonHeight);
      end;
      with TButton.Create(Form) do
      begin
        Parent := Form;
        Caption := lng_ButtonCaptions[mbCancel];
        ModalResult := mrCancel;
        Cancel := True;
        SetBounds(MulDiv(122, DialogUnits.X, 4), ButtonTop, ButtonWidth, ButtonHeight);
        Form.ClientHeight := Top + Height + 13;
      end;
      if ShowModal = mrOk then
      begin
        if Assigned(sItems) and Assigned(cEdit) then
        begin
          if Fixed then
            Value:=cEdit.Items[cEdit.ItemIndex]
          else
            Value := cEdit.Text
        end
        else
          Value := Edit.Text;
        Result := True;
      end;
    finally
      Form.Free;
    end;
end;

function InputQueryMemoLM(const ACaption, APrompt: string;
  var Value: string; mWidth:Integer=232; mHeight:Integer=232): Boolean;
var
  Form: TForm;
  Prompt: TLabel;
  Edit: TMemo;
  DialogUnits: TPoint;
  ButtonTop, ButtonWidth, ButtonHeight: Integer;
begin
  Result := False;
  Form := TForm.Create(Application);
  with Form do
    try
      Canvas.Font := Font;
      DialogUnits := _GetAveCharSize(Canvas);
      BorderStyle := bsDialog;
      Caption := ACaption;
      {$IFDEF D12}
      PopupMode := pmAuto;
      {$ENDIF D12}
      Position := poScreenCenter;
      Prompt := TLabel.Create(Form);
      with Prompt do
      begin
        Parent := Form;
        Caption := APrompt;
        Left := MulDiv(8, DialogUnits.X, 4);
        Top := MulDiv(8, DialogUnits.Y, 8);
        Constraints.MaxWidth := MulDiv(232, DialogUnits.X, 4);
        ShowAccelChar:=False;
        WordWrap := True;
      end;
      Edit := TMemo.Create(Form);
      with Edit do
      begin
        Parent := Form;
        Left := Prompt.Left;
        Top := Prompt.Top + Prompt.Height + 5;
        Width := MulDiv(mWidth, DialogUnits.X, 4);
        Height := MulDiv(trunc(mHeight/1.78), DialogUnits.Y, 8);
        Wordwrap := True;
        WantTabs := False;
        WantReturns := False;
        Lines.Text := Value;
        SelectAll;
      end;
      ClientWidth := Edit.Left + Edit.Width + 13;
      ButtonTop := Edit.Top + Edit.Height + 15;

      ButtonWidth := MulDiv(50, DialogUnits.X, 4);
      ButtonHeight := MulDiv(14, DialogUnits.Y, 8);
      with TButton.Create(Form) do
      begin
        Parent := Form;
        Caption := lng_ButtonCaptions[mbOK];
        ModalResult := mrOk;
        Default := True;
        SetBounds((Form.ClientWidth div 2)-ButtonWidth-2, ButtonTop, ButtonWidth, ButtonHeight);
      end;
      with TButton.Create(Form) do
      begin
        Parent := Form;
        Caption := lng_ButtonCaptions[mbCancel];
        ModalResult := mrCancel;
        Cancel := True;
        SetBounds((Form.ClientWidth div 2)+2, ButtonTop, ButtonWidth, ButtonHeight);
        Form.ClientHeight := Top + Height + 13;
      end;
      if ShowModal = mrOk then
      begin
        Value := Edit.Lines.Text;
        Result := True;
      end;
    finally
      Form.Free;
    end;
end;

end.
